/*
 * Copyright (c) 2021 Digital Asset (Switzerland) GmbH and/or its affiliates
 *
 * Proprietary code. All rights reserved.
 */

-- Stores the received sequencer messages
create table sequenced_events
(
    -- discriminate between different users of the sequenced events tables
    client            NVARCHAR2(300) not null,
    -- Proto serialized signed message
    sequenced_event   BLOB, -- allowing null, as ignored events may have an empty body, which is treated as NULL in Oracle.
    -- Explicit fields to query the messages, which are stored as blobs
    type              VARCHAR2(3)  not null,
    -- Timestamp of the time of change in microsecond precision relative to EPOCH
    ts                NUMBER          not null,
    -- Sequencer counter of the time of change
    sequencer_counter NUMBER          not null,
    -- serialized trace context associated with the event
    trace_context     BLOB            not null,
    -- flag to skip problematic events
    ignore            NUMBER(1)       not null,
    -- The sequencer ensures that the timestamp is unique
    constraint pk_sequenced_events primary key (ts, client)
);

create unique index idx_sequenced_events_sequencer_counter on sequenced_events(sequencer_counter, client);

-- message type: deliver event, heartbeat
ALTER table sequenced_events
    ADD CONSTRAINT event_type_constraint check
        (type in ('del', 'hb', 'err', 'ign'));

