/*
 * Copyright (c) 2021 Digital Asset (Switzerland) GmbH and/or its affiliates
 *
 * Proprietary code. All rights reserved.
 */

-- Maintains the number of active contracts for a given key (by hash) as a journal
create table contract_key_journal
(
    -- For uniformity with other tables, we store the domain ID with the allocation count,
    -- even though the contract key journal is currently used only for domains with unique contract key (UCK) semantics
    -- and participants can connect only to one UCK domain.
    domain_id         NVARCHAR2(300) not null,
    contract_key_hash NVARCHAR2(300) not null,
    status            varchar(300)   not null,
    -- Timestamp of the time of change in microsecond precision relative to EPOCH
    ts                NUMBER          not null,
    -- Request counter of the time of change
    request_counter   NUMBER          not null,
    constraint pk_contract_key_journal primary key (contract_key_hash, domain_id, ts, request_counter)
);

ALTER table contract_key_journal
    ADD CONSTRAINT contract_key_status_constraint check
        (status in ('unassigned', 'assigned'));

CREATE index contract_key_journal_dirty_request_reset_idx ON contract_key_journal (domain_id, ts, request_counter);
