/*
 * Copyright (c) 2021 Digital Asset (Switzerland) GmbH and/or its affiliates
 *
 * Proprietary code. All rights reserved.
 */

-- oracle events table (differs from h2 and postgres in the recipients array definition)
create type INTEGER_ARRAY as VARRAY (100) of INTEGER;
/
create table sequencer_events
(
    ts                NUMBER constraint pk_sequencer_events primary key,
    node_index        NUMBER(1) not null,
    -- single char to indicate the event type: D for deliver event, H for heartbeat, E for deliver error
    event_type        CHAR(1)         not null
        constraint event_type_enum check (event_type = 'D' or event_type = 'H' or event_type = 'E'),
    message_id        NVARCHAR2(300) null,
    sender            INTEGER         null,
    -- null if event goes to everyone, otherwise specify member ids of recipients
    recipients        INTEGER_ARRAY   null,
    -- null if the event is a heartbeat
    -- intentionally not creating a foreign key here for performance reasons
    payload_id        NUMBER         null,
    -- optional signing timestamp for deliver if supplied
    signing_timestamp NUMBER          null,
    error_message     CLOB null,
    -- trace context associated with the event
    trace_context     BLOB            not null
);