/*
 * Copyright (c) 2021 Digital Asset (Switzerland) GmbH and/or its affiliates
 *
 * Proprietary code. All rights reserved.
 */

create table command_deduplication (
  -- hash of the change ID (application_id + command_id + act_as) as a hex string
  change_id_hash NVARCHAR2(300) constraint pk_command_deduplication primary key,

  -- the application ID that requested the change
  application_id NVARCHAR2(300) not null,
  -- the command ID
  command_id NVARCHAR2(300) not null,
  -- the act as parties serialized as a Protobuf blob
  act_as BLOB not null,

  -- the highest offset in the MultiDomainEventLog that yields a completion event with a definite answer for the change ID hash
  offset_definite_answer NUMBER not null,
  -- the publication time of the offset_definite_answer, measured in microseconds relative to EPOCH
  publication_time_definite_answer NUMBER not null,
  submission_id_definite_answer NVARCHAR2(300), -- always optional
  trace_context_definite_answer BLOB not null,

  -- the highest offset in the MultiDomainEventLog that yields an accepting completion event for the change ID hash
  -- always at most the offset_definite_answer
  -- null if there have only been rejections since the last pruning
  offset_acceptance NUMBER,
  -- the publication time of the offset_acceptance, measured in microseconds relative to EPOCH
  publication_time_acceptance NUMBER,
  submission_id_acceptance NVARCHAR2(300),
  trace_context_acceptance BLOB
);
create index idx_command_dedup_offset on command_deduplication(offset_definite_answer);

create table command_deduplication_pruning (
  client NUMBER constraint pk_command_deduplication_pruning primary key, -- dummy field to enforce at most one row

  -- The highest offset pruning has been started at.
  pruning_offset NUMBER not null,
  -- An upper bound to publication times of pruned offsets.
  publication_time NUMBER not null
);
