/*
 * Copyright (c) 2021 Digital Asset (Switzerland) GmbH and/or its affiliates
 *
 * Proprietary code. All rights reserved.
 */

-- Maintains the status of contracts
create table active_contracts
(
    -- As a participant can be connected to multiple domains, the active contracts are stored under a domain id.
    domain_id        NVARCHAR2(300) not null,
    contract_id      NVARCHAR2(300) not null,
    change           VARCHAR2(100)   not null,
    operation        VARCHAR2(100)   not null,
    -- UTC timestamp of the time of change in microsecond precision relative to EPOCH
    ts               NUMBER          not null,
    -- Request counter of the time of change
    request_counter  NUMBER          not null,
    -- optional remote domain id in case of transfers
    remote_domain_id NVARCHAR2(300),
    constraint pk_active_contracts primary key (contract_id, ts, request_counter, change, domain_id)
);

-- change type: activation [create, transfer-in], deactivation [archive, transfer-out]
ALTER table active_contracts
    ADD CONSTRAINT active_contracts_change_constraint check
        (change in ('deactivation', 'activation'));

-- The specific operation type that introduced a contract change.
ALTER table active_contracts
    ADD CONSTRAINT active_contracts_operation_constraint check
        (operation in ('create', 'transfer-in', 'archive', 'transfer-out'));

CREATE index active_contracts_dirty_request_reset_idx ON active_contracts (request_counter, domain_id);

CREATE index active_contracts_contract_id_idx ON active_contracts (contract_id);

CREATE index active_contracts_ts_domain_id_idx ON active_contracts (ts, domain_id);
